unit ICAboutWhats;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module drives the user items in the About Box.  The basic idea is to install
	an Dialog Manager update routine for each item that draws the item, and to handle
	clicks in the About Box by launching specific URLs.
*)

interface

	uses
		ICWindowGlobals;

	(* ***** Standard What Interface ***** *)
	
	(* See the comments in ICWindowGlobals. *)

	function WhatInitAbout : OSStatus;
	function WhatOpenAbout (wt: WindowType; item: integer): OSStatus;
	function WhatClickAbout (wt: WindowType; item: integer; event: EventRecord): OSStatus;

	(* ***** Layer Breaking Interface ***** *)

	procedure AdjustAboutMenu;
		(* The About box is at the front; adjust the Edit menu accordingly. *)
	
	procedure DoAboutMenu(item : integer);
		(* An item was chosen from the item while the About box was at the front. *)

implementation

	uses
		Scrap,
		
		ICResources, 
		ICStringLists,
		ICDialogs, 
		ICMiscSubs, 
		ICGlobals, 
		ICWindowGlobals,
		ICDocument;

	var
		gAboutBoxUserItemUpdateUPP : UserItemUPP;

	(* ***** Layer Breaking Stuff ***** *)

	procedure AdjustAboutMenu;
		(* See comment in interface part. *)
		var
			editMenu : MenuHandle;
	begin
		editMenu := GetMenuHandle(mEdit);
		SetMenuItemEnable(editMenu, iUndo, false);
		SetMenuItemEnable(editMenu, iCut, false);
		SetMenuItemEnable(editMenu, iCopy, true);
		SetMenuItemEnable(editMenu, iPaste, false);
		SetMenuItemEnable(editMenu, iClear, false);
		SetMenuItemEnable(editMenu, iSelectAll, false);
	end; (* AdjustAboutMenu *)
	
	procedure DoAboutMenu(item : integer);
		(* See comment in interface part. *)
		var
			junk : longint;
			urlStr : Str255;
	begin
		if item = iCopy then begin
			urlStr := GetIndStr(rAboutBoxStrings, ditAboutBoxFAQURL);
			delete(urlStr, 1, pos('<', urlStr) - 1);
			junk := ZeroScrap;
			junk := PutScrap(length(urlStr), 'TEXT', @urlStr[1])
		end; (* if *)
	end; (* DoAboutMenu *)

	(* ***** Utilities ***** *)

	function LaunchSomeSillyURL(urlString: Str255): ICError;
		(* Call IC to open the URL that's passed in. Note that the actual URL
			is embedded somewhere in urlString and so this routine has to:
			
			1. pass in startIndex and endIndex to IC to get it to search out the URL
			2. do some parsing when the URL launch fails and it needs to find
				the scheme in order to put up the error dialog.
			
			We have to use the CreateSpecifiedInstance routine (as opposed
			to just using the default document instance) because this code
			can be executed when there's no document open.
		*)
		var
			err : ICError;
			startIndex : longint;
			endIndex: longint;
			junk : integer;
			urlScheme : Str255;
			alertID : integer;
			inst : ICInstance;
	begin
		startIndex := length(urlString) div 2;
		endIndex := startIndex;
		err := CreateSpecifiedInstance(inst);
		if err = noErr then begin
			err := ICLaunchURL(inst, '', @urlString[1], length(urlString), 
											startIndex, endIndex);
			DestroySpecifiedInstance(inst);
		end; (* if *)
		
		(* If we get a known error code, put up a special error dialog rather
			than leaving it to the default error handler.
		*)
		if (err = icPrefNotFoundErr) or (err = afpItemNotFound) then begin

			(* Remove everything up to and including the '<' in urlString. *)
			delete(urlString, 1, pos('<', urlString));

			(* Find urlScheme by copying out everything up to the colon. *)
			urlScheme := copy(urlString, 1, pos(':', urlString) - 1);
			ParamText(urlScheme, '', '', '');
			
			(* Choose your alert message. *)
			if err = icPrefNotFoundErr then begin
				alertID := rLaunchURLErrorAlert;
			end else begin
				alertID := rLaunchURLError2Alert;
			end; (* if *)

			(* Display the dialog and return noErr to stop the default error handler
				also displaying a dialog.
			*)
			junk := StopAlert(alertID, gOKModalFilterUPP);
			err := noErr;
		end; (* if *)
		LaunchSomeSillyURL := err;
	end; (* LaunchSomeSillyURL *)

	(* ***** Drawing ***** *)

	procedure AboutBoxUserItemUpdateProc (dlg: DialogPtr; item: integer);
		(* This routine is called by the Dialog Manager to update the user items
			in the About Box.
		*)
		var
			itemRect: Rect;
	begin
		GetDItemRect(dlg, item, itemRect);
		case item of
			ditAboutBoxIcon: 
				DrawIcon(rApplicationIcon, itemRect, false);
			ditAboutBoxVersion: 
				DrawStyledString(dlg, item, concat(GetIndStr(rAboutBoxStrings, item), gApplicationVersion.shortVersion));
			otherwise
				DrawStyledString(dlg, item, GetIndStr(rAboutBoxStrings, item));
		end; (* case *)
	end; (* AboutBoxUserItemUpdateProc *)

	(* ***** Standard What Interface ***** *)

	(* ***** What Init ***** *)

	function WhatInitAbout : OSStatus;
	begin
		gAboutBoxUserItemUpdateUPP := NewUserItemProc(@AboutBoxUserItemUpdateProc);
		WhatInitAbout := noErr;
	end; (* WhatInitAbout *)

	(* ***** What Open ***** *)

	function WhatOpenAbout (wt: WindowType; item: integer): OSStatus;
		(* See the comments in ICWindowGlobals. *)
		var
			itemKind : integer;
	begin
		(* Install the user item update proc if we're a user item. *)
		GetDItemKind(gWindows[wt].window, item, itemKind);
		if band(itemKind, $07F) = userItem then begin
			SetDItemHandle(gWindows[wt].window, item, gAboutBoxUserItemUpdateUPP);
		end; (* if *)
		WhatOpenAbout := noErr;
	end; (* WhatOpenAbout *)

	(* ***** What Click ***** *)

	function WhatClickAbout (wt: WindowType; item: integer; event: EventRecord): OSStatus;
		(* See the comments in ICWindowGlobals. *)
		var
			itemHit:integer;
	begin
		{$unused event}
		if TrackDItem(gWindows[wt].window, item) then begin

			(* If they hit the "mail to IC support" button, put up a dialog
				asking them if they really want to do that, giving them
				the option of looking at the FAQ.  Then adjust the item
				number appropriately.
			*)
			if item = ditAboutBoxMailToURL then begin
				itemHit := NoteAlert(rMailConfirmAlert, gOKCancelModalFilter);
				case itemHit of
					ditFAQ:
						item := ditAboutBoxFAQURL;
					ditCancel:
						item := 0;
					ditEmail:
						item := ditAboutBoxMailToURL;
				end; (* case *)
			end; (* if *)
			if item <> 0 then begin
				DisplayError(acLaunchHelper, LaunchSomeSillyURL(GetIndStr(rAboutBoxStrings, item)));
			end; (* if *)
		end; (* if *)
		WhatClickAbout := noErr;
	end; (* WhatClickAbout*)

end. (* ICAboutWhats *)
