unit ICEvents;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module implements the inside of the application's main event loop. It implements the
	bulk of the standard Macintosh "get this event, Handle this event in the default way" stuff,
	and calls out the ICWindows to do the real work.  There is not a lot of excitement here.
*)

interface

	procedure HandleAnEvent;
		(* Implements the application's main event loop. *)

implementation

	uses
		EPPC,
		ToolUtils,
		AppleEvents,
		DiskInit,
		Windows,
		Devices, 

		ICResources,
		ICGlobals,
		ICMiscSubs,
		ICStringLists,
		ICWindowGlobals,
		ICWindows,
		ICDocument,
		ICInstall;

	procedure DoDiskEvent (message: longint);
		(* Handle bad disk inserted events in the usual way. *)
		var
			pt: Point;
			junkErr: OSStatus;
	begin
		if (hiwrd(message) <> noErr) then begin
			pt.h := ((qd.screenBits.bounds.right - qd.screenBits.bounds.left - 304) div 2);
			pt.v := ((qd.screenBits.bounds.bottom - qd.screenBits.bounds.top - 156) div 3);
			InitCursor;
			junkErr := DIBadMount(pt, message);
		end; (* if *)
	end; (* DoDiskEvent *)

	procedure DoActivateDeactivate (window: WindowPtr; activate: Boolean);
		(* Handle activate/deactivate events by highlighting the window appropriately and also
			passing the event along to ICWindows, which does the right thing for our TextEdit fields.
		*)
	begin
		if activate then begin
			SelectWindow(window);
		end; (* if *)
		WindowActivateDeactivate(window, activate);
	end; (* DoActivateDeactivate *)

	procedure DoSuspendResume (resume: Boolean);
		(* Handle suspend and resume events by activating/deactivating the front window. *)
	begin
		gInForeground := resume;
		if FrontWindow <> nil then begin
			DoActivateDeactivate(FrontWindow, resume);
		end; (* if *)
		InitCursor;
	end; (* DoSuspendResume *)
	
	function DoCloseWindow(wp : WindowPtr) : OSStatus;
		var
			err : OSStatus;
	begin
		err := noErr;
		if wp <> nil then begin
			if WindowPeek(wp)^.windowKind < 0 then begin
				CloseDeskAcc(WindowPeek(wp)^.windowKind);
			end else begin
				err := DoCloseDocumentWindow(wp);
			end; (* if *)
		end; (* if *)
		DoCloseWindow := err;
	end; (* DoCloseWindow *)

	procedure AdjustMenus;
		(* Adjust all the menus in preparation for a MenuSelect, MenuKey, or DimMenuTitles operation. *)
		var
			editMenu : MenuHandle;
			wp : WindowPtr;
	begin
		(* File *)
		SetMenuItemEnable(GetMenuHandle(mFile), iClose, IsDocumentOpen or (MapWindowPtrToWindowType(FrontWindow) = WT_About));
		SetMenuItemEnable(GetMenuHandle(mFile), iSave, IsDocumentDirty or IsDocumentNew);
		SetMenuItemEnable(GetMenuHandle(mFile), iSaveAs, IsDocumentOpen);
		(* Edit *)
		wp := FrontWindow;
		if (wp <> nil) & (WindowPeek(wp)^.windowKind < 0) then begin
			(* If we have a DA at the front, turn on the standard edit commands. *)
			editMenu := GetMenuHandle(mEdit);
			SetMenuItemEnable(editMenu, iUndo, false);
			SetMenuItemEnable(editMenu, iCut, true);
			SetMenuItemEnable(editMenu, iCopy, true);
			SetMenuItemEnable(editMenu, iPaste, true);
			SetMenuItemEnable(editMenu, iClear, true);
			SetMenuItemEnable(editMenu, iSelectAll, false);
		end else begin
			WindowsAdjustEditMenu;
		end; (* if *)
		(* Install *)
		AdjustInstallMenu;
		(* Windows *)
		if IsDocumentOpen then begin
			SetMenuItemText(GetMenuHandle(mWindows), iMainWindowName, GetDocumentName);
		end else begin
			SetMenuItemText(GetMenuHandle(mWindows), iMainWindowName, GetIndStr(rMiscStrings, strFirstWindowMenuItemText));
		end; (* if *)
	end; (* AdjustMenus *)

	procedure DimMenuTitles;
		(* Called periodically by the main event loop to enable/disable the titles of menus. This routine
			works by first adjusting the menus to reflect their current state, then seeing whether the
			menu title state matches the state of the items, and setting it otherwise. We also keep track
			of whether any of the menu titles have changed to decide whether the menu bar needs redrawing.
		*)
		var
			menuH: MenuHandle;
			flags: longint;
			i: integer;
			wasTitleEnabled: Boolean;
			shouldTitleBeEnabled: Boolean;
			menuBarNeedRedraw: Boolean;
	begin
		AdjustMenus;

		(* Walk the Edit menu to see if any menu items are set. *)
		menuH := GetMenuHandle(mEdit);
		flags := menuH^^.enableFlags;
		wasTitleEnabled := btst(flags, 0);
		shouldTitleBeEnabled := false;
		for i := 1 to CountMItems(menuH) do begin
			if btst(flags, i) then begin
				shouldTitleBeEnabled := true;
				leave;
			end; (* if *)
		end; (* for *)
		SetMenuItemEnable(menuH, 0, shouldTitleBeEnabled);
		menuBarNeedRedraw := (shouldTitleBeEnabled <> wasTitleEnabled);
		
		(* Do the same for the Windows menu. The state of all the items on the menu follows
			whether a document is open, so we don't need to actually loop.
		*)
		menuH := GetMenuHandle(mWindows);
		flags := menuH^^.enableFlags;
		wasTitleEnabled := btst(flags, 0);
		shouldTitleBeEnabled := IsDocumentOpen;
		SetMenuItemEnable(menuH, 0, shouldTitleBeEnabled);
		menuBarNeedRedraw := menuBarNeedRedraw or (shouldTitleBeEnabled <> wasTitleEnabled);
		
		(* Redraw the menu bar if necessary. *)
		if menuBarNeedRedraw then begin
			DrawMenuBar;
		end; (* if *)
	end; (* DimMenuTitles *)

	procedure DoMenu (menuAndItem : longint);
		(* Handle menu events. Basically this is the standard "Handle the boring events here and dispatch
			the rest to the appropriate place."
		*)
		var
			startTime : longint;
			menu : integer;
			item : integer;
			deskAccessoryName: Str255;
			junk: OSStatus;
	begin
		startTime := TickCount;
		menu := hiwrd(menuAndItem);
		item := lowrd(menuAndItem);
		case menu of
			mApple:  begin
				case item of
					iAbout: 
						DisplayError(acOpenWindow, WindowOpen(WT_About));
					otherwise begin
						GetMenuItemText(GetMenuHandle(mApple), item, deskAccessoryName);
						junk := OpenDeskAcc(deskAccessoryName);
					end
				end; (* case *)
			end;
			mFile:  begin
				case item of
					iNew:
						DisplayError(acNewDocument, DoNewDocument);
					iOpen: 
						DisplayError(acOpenDocument, DoOpenDocumentUsingStandardFile);
					iOpenInternetPreferences: 
						DisplayError(acOpenDocument, DoOpenDefaultDocument);
					iClose: 
						DisplayError(acCloseWindow, DoCloseWindow(FrontWindow));
					iSave: 
						DisplayError(acSave, DoSaveDocument);
					iSaveAs: 
						DisplayError(acSave, DoSaveAsDocument);
					iQuit: 
						DisplayError(acQuit, DoQuit);
					otherwise
						;
				end; (* case *)
			end;
			mEdit:  begin
				case item of
					iUndo, iCut, iCopy, iPaste, iClear, iSelectAll:
						if (item > iClear) | not SystemEdit(item - 1) then begin
							WindowsDoEditMenu(item);
						end; (* if *)
					otherwise
						;
				end;
			end;
			mInstall:
				DoInstallMenu(item);
			mWindows:
				DisplayError(acOpenWindow, WindowOpen(WindowType(ord(WT_Main) + item - 1)));
			otherwise
				;
		end; (* case *)
		if not gQuitNow then begin

			(* For very fast menu commands, hang around until the user has time
				to actually see the menu highlighting.
			*)
			while TickCount < (startTime + 6) do begin
				Delay(1, junk);
			end; (* while *)
			
			HiliteMenu(0);
		end; (* if *)
	end; (* DoMenu *)

	procedure DoMainClick (event: EventRecord; wp: WindowPtr; clickCode: integer);
		(* Handle a mouseDown event that hasn't been fielded by the Dialog Manager. Basically
			this translates to clicks in the menu bar and close box, dragging windows, selecting
			non-frontmost windows.
		*)
	begin
		(* Then go and do each type of event. *)
		case clickCode of
			inMenuBar:  begin
				AdjustMenus;
				DoMenu(MenuSelect(event.where));
			end;
			
			inDrag:  begin
				SetPort(wp);			(* I'm not quite sure what this is doing here. *)
				DragWindow(wp, event.where, qd.screenBits.bounds);
			end;
			
			inGoAway:  begin
				if TrackGoAway(wp, event.where) then begin
					DisplayError(acCloseWindow, DoCloseWindow(wp));
				end; (* if *)
			end;
			
			inContent:  begin
				if (wp <> nil) & (wp <> FrontWindow) then begin
					SelectWindow(wp);
				end; (* if *)
			end;
			
			inSysWindow: 
				SystemClick(event, wp);

			inGrow: 
				;
			inZoomIn, inZoomOut: 
				;
			otherwise
				(* do nothing *)
		end; (* case *)
	end; (* DoMainClick *)

	procedure HandleAnEvent;
		(* Get an event and Handle it! *)
		var
			junkBool: Boolean;
			event: EventRecord;
			eventPending: Boolean;
			clickCode: integer;
			wp: WindowPtr;
			menuAndItem: longint;
			junk: OSStatus;
			itemHit: integer;
	begin
		DimMenuTitles;
		junkBool := WaitNextEvent(everyEvent, event, 10, nil);
		WindowsIdle;
		eventPending := true;
		
		(* Prehandle keyboard events. *)
		if eventPending & ((event.what = keyDown) | (event.what = autoKey)) then begin
			(* First command keys... *)
			if band(event.modifiers, cmdKey) <> 0 then begin
				AdjustMenus;
				menuAndItem := MenuKey(chr(band(event.message, charCodeMask)));
				if menuAndItem <> 0 then begin
					DoMenu(menuAndItem);
					eventPending := false;
				end; (* if *)
			end; (* if *)
			(* ... then prehandle normal keys. *)
			if eventPending & WindowsEarlyHandleKey(event) then begin
				eventPending := false;
			end; (* if *)
		end; (* if *)
		
		(* Hand the event off to the Dialog Manager. *)
		if eventPending & IsDialogEvent(event) then begin
			if DialogSelect(event, wp, itemHit) then begin
				WindowItemClick(wp, event, itemHit);
				eventPending := false;
			end; (* if *)
		end; (* if *)
		
		(* Do a classic main event loop case statement. *)
		if eventPending then begin
			case event.what of
				mouseDown:  begin
					clickCode := FindWindow(event.where, wp);
					if wp = nil then begin
						wp := FrontWindow;
					end;
					DoMainClick(event, wp, clickCode);
				end;

				keyDown, autoKey:
					WindowsDoKey(event);

				updateEvt: 
					(* do nothing -- all update events are handled by the Dialog Manager *) ;

				activateEvt: 
					DoActivateDeactivate(WindowPtr(event.message), odd(event.modifiers));

				osEvt: 
					if band(bsr(event.message, 24), $FF) = suspendResumeMessage then begin
						DoSuspendResume(band(event.message, resumeFlag) <> 0);
					end;

				diskEvt: 
					DoDiskEvent(event.message);

				kHighLevelEvent: 
					if gHasAppleEvents then begin
						junk := AEProcessAppleEvent(event);
					end; (* if *)
				otherwise
			 (* do nothing *)
			end; (* case *)
		end; (* if *)
	end; (* HandleAnEvent *)

end. (* ICEvents *)