unit ICInstall;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module implements the application's Install menu, as well as containing
	code that's called at startup time to ensure that the right version of the
	component is installed.
	
	The component is stored in a 'ThNg' resource in the application. This resource
	holds a byte image of the resource fork of the Internet Config Extension,
	as created by the MPW build process.
*)

interface

	uses
		Types,
		Files;

	(* ***** Install Menu Handling ***** *)
	
	procedure AdjustInstallMenu;
		(* This routine adjusts the install menu, ie enables and disables items to
			reflect the current status of the installation.
		*)
	
	procedure DoInstallMenu (item: integer);
		(* This routine dispatches menu commands from the install menu. *)
	
	function InitializeComponentInstallation: OSStatus;
		(* This routine is called at startup. It goes to great lengths to ensure
			that the right version of the Internet Config component is installed
			and available for use.
		*)

	function IsExtensionInstalled (var extensionHome : FSSpec): Boolean;
		(* See comment in implementation part. This routine is exported
			so that the File Map What can find the extension and extract
			the default file mapping database from its resource fork.
		*)

	function SaveComponentToFile (destFile: FSSpec): OSStatus;

implementation

	uses
		Folders,
		Components,
		Resources,
		Dialogs,

		ICTypes,
		ICCAPI, 
		ICAPI, 

		ICResources, 
		ICStringLists, 
		ICMiscSubs, 
		ICGlobals, 
		ICStandardFile, 
		ICDialogs;

	var
		gApplicationVersionAsLongint: longint;
		gInstalledComponentVersion: longint;
		(* A local cache of the application and component versions, stored as longints
			to aid in comparisons. These should really be stored as unsigned longints
			(UInt32) but the current support for unsigned types in Metrowerks Pascal
			is bleeding edge, and I don't want to provoke it.  It doesn't really make any
			difference until we get to version 128 of IC, something that will most probably
			take some time.
		*)

	function GetVersionFromCurResFile: longint;
		(* Get the version number out of the current resource file. *)
		var
			versh: VersRecHndl;
	begin
		GetVersionFromCurResFile := 0;
		versh := VersRecHndl(Get1Resource('vers', 1));
		if versh <> nil then begin
			GetVersionFromCurResFile := longint(versh^^.numericVersion);
		end; (* if *)
	end; (* GetVersionFromCurResFile *)

	function GetRegisteredComponentVersion: longint;
		(* Get the version of the current registered Internet Config component. *)
		var
			inst: ComponentInstance;
			junk: OSStatus;
			refnum: integer;
			result : longint;
	begin
		result := 0;
		if gHasComponentManager then begin
			inst := OpenDefaultComponent(internetConfigurationComponentType, internetConfigurationComponentSubType);
			if inst <> nil then begin
				refnum := OpenComponentResFile(Component(inst));
				if refnum <> -1 then begin
					result := GetVersionFromCurResFile;
					CloseResFile(refnum);
				end; (* if *)
				junk := CloseComponent(inst);
			end; (* if *)
		end; (* if *)
		GetRegisteredComponentVersion := result;
	end; (* GetRegisteredComponentVersion *)

	function IsExtensionInstalled (var extensionHome : FSSpec): Boolean;
		(* Returns true if the Internet Config Extension is present in the place it should
			be. extensionHome is set to be either the place we found the extension (if we return
			true), or the best place for the extension (if we return false).
		*)
		var
			err: OSStatus;
			ndx: integer;
			found: Boolean;
			cpb: CInfoPBRec;
			info: FInfo;
	begin
		(* Find the extension folder, using the glue for the System 6 case. *)
		err := FindFolder(kOnSystemDisk, kExtensionFolderType, true, 
											extensionHome.vRefNum, extensionHome.parID);
		(* Complete the FSSpec using the default name. *)
		extensionHome.name := GetIndStr(rMiscStrings, strInternetConfigExtension);
		if err = noErr then begin
			(* Look for the extension in the default place. *)
			err := HGetFInfo(extensionHome.vRefNum, extensionHome.parID, extensionHome.name, info);
			if err = noErr then begin
				(* If we find it, life we cool, we can just return. *)
				(* do nothing *)
			end else begin
				(* If we don't find it, go searching the Extensions folder for it. *) 
				found := false;
				ndx := 1;
				repeat
					cpb.ioNamePtr := @extensionHome.name;
					cpb.ioVRefNum := extensionHome.vRefNum;
					cpb.ioDirID := extensionHome.parID;
					cpb.ioFDirIndex := ndx;
					err := PBGetCatInfoSync(@cpb);
					if err = noErr then begin
						found := (cpb.ioFlFndrInfo.fdType = 'thng') and (cpb.ioFlFndrInfo.fdCreator = ICcreator);
					end; (* if *)
					ndx := ndx + 1;
				until found or (err <> noErr);
				
				(* If we still haven't found it, set the FSSpec back to the default location. *)
				if not found then begin
					extensionHome.name := GetIndStr(rMiscStrings, strInternetConfigExtension);
					err := fnfErr;
				end; (* if *)
			end; (* if *)
		end; (* if *)
		IsExtensionInstalled := (err = noErr);
	end; (* IsExtensionInstalled *)

	procedure UpdateInstalledComponentVersionGlobal (var componentFileSpec: FSSpec);
		(* Update the gInstalledComponentVersion to reflect the version
			of the file sitting in the Extensions folder. If this version is
			greater than 0, componentFileSpec will point to the installed file.
			If it's equal to 0,  componentFileSpec will point to where the file
			should be installed.
		*)
		var
			ref: integer;
			err: OSStatus;
	begin
		gInstalledComponentVersion := 0;
		if IsExtensionInstalled(componentFileSpec) then begin
			ref := HOpenResFile(componentFileSpec.vRefNum, componentFileSpec.parID,
												componentFileSpec.name, fsRdPerm);
			err := ResError;
			if err = noErr then begin
				gInstalledComponentVersion := GetVersionFromCurResFile;
				CloseResFile(ref);
			end; (* if *)
		end; (* if *)
	end; (* UpdateInstalledComponentVersionGlobal *)

	function SaveComponentToFile (destFile: FSSpec): OSStatus;
		(* Saves the Internet Config Extension to the destFile, overwriting
			anything that's there.
		*)
		var
			err: OSStatus;
			extensionResourceForkHandle: Handle;
			numberOfBytesToWrite : longint;
			ref: integer;
			junk: OSStatus;
	begin
		(* Get the image of the extension's resource fork. *)
		err := noErr;
		extensionResourceForkHandle := GetResource('ThNg', rInternetConfigComponent);
		if extensionResourceForkHandle = nil then begin
			err := resNotFound;
		end; (* if *)
		
		(* Save it out the destFile. *)
		if err = noErr then begin
			HNoPurge(extensionResourceForkHandle);
			junk := HCreate(destFile.vRefNum, destFile.parID, destFile.name, ICcreator, 'thng');
			err := HOpenRF(destFile.vRefNum, destFile.parID, destFile.name, fsRdWrPerm, ref);
			if err = noErr then begin
				err := SetEOF(ref, GetHandleSize(extensionResourceForkHandle));
				if err = noErr then begin
					numberOfBytesToWrite := GetHandleSize(extensionResourceForkHandle);
					err := FSWrite(ref, numberOfBytesToWrite, extensionResourceForkHandle^);
				end; (* if *)
				junk := FSClose(ref);
			end; (* if *)
			HPurge(extensionResourceForkHandle);
		end; (* if *)
		
		(* Now update gInstalledComponentVersion. We do this for two reasons:
			firstly, we might be being called by JustInstallICComponent in which
			case we need to update the version because it will be the version of
			the installed component. Secondly, we might be being called from
			SaveICComponent and the user might have chosen to save the
			component into the Extensions folder. So we need to call
			UpdateInstalledComponentVersionGlobal it in both cases, which
			is what it's doing in this routine.
		*)
		UpdateInstalledComponentVersionGlobal(destFile);
		
		SaveComponentToFile := err;
	end; (* SaveComponentToFile *)

	function RegisterFile (componentFile: FSSpec): OSStatus;
		(* Register the components in the componentFile. *)
		var
			ref: integer;
			err, err2: OSStatus;
			numRegistered: longint;
	begin
		ref := HOpenResFile(componentFile.vRefNum, componentFile.parID, componentFile.name, fsRdPerm);
		err := ResError;
		if err = noErr then begin
			numRegistered := RegisterComponentResourceFile(ref, registerComponentGlobal);
			if numRegistered > 0 then begin
				err := noErr;
			end else begin
				err := numRegistered;
			end; (* if *)
			CloseResFile(ref);
			err2 := ResError;
			if err = noErr then begin
				err := err2;
			end; (* if *)
		end; (* if *)
		RegisterFile := err;
	end; (* RegisterFile *)

	function JustInstallICComponent: OSStatus;
		(* This routine just installs the Internet Config Extension and registers
			the components contained therein. It is called by both the code
			handling the Install Extension menu command, and the code
			that's executed at application startup.
		*)
		var
			err: OSStatus;
			whereToInstall: FSSpec;
	begin
		UpdateInstalledComponentVersionGlobal(whereToInstall);
		err := SaveComponentToFile(whereToInstall);
		if err = noErr then begin
			if gHasComponentManager then begin
				err := RegisterFile(whereToInstall);
			end; (* if *)
		end; (* if *)
		JustInstallICComponent := err;
	end; (* JustInstallICComponent *)

	(* ***** Install Menu Handling ***** *)

	procedure AdjustInstallMenu;
		var
			menuH : MenuHandle;
	begin
		menuH := GetMenuHandle(mInstall);
		SetMenuItemEnable(menuH, iInstallExtension, gHasComponentManager & (gInstalledComponentVersion <> gApplicationVersionAsLongint));
		SetMenuItemEnable(menuH, iRemoveExtension, gInstalledComponentVersion > 0);
	end; (* AdjustInstallMenu *)

	function SaveICComponent: OSStatus;
		(* Implements the functionality of the Save Extension command on
			the Install menu.
		*)
		var
			err: OSStatus;
			whereToSave: FSSpec;
	begin
		err := ICStandardPutFile(GetIndStr(rMiscStrings, strSaveExtensionPrompt), 
												GetIndStr(rMiscStrings, strInternetConfigExtension), 
												whereToSave);
		if err = noErr then begin
			err := SaveComponentToFile(whereToSave);
		end; (* if *)
		SaveICComponent := err;
	end; (* SaveICComponent *)
	
	function InstallICComponent: OSStatus;
		(* Implements the functionality of the Install Extension command on
			the Install menu. It finds out the version of the currently installed
			component, determines whether it's older or new then the one
			we have in our resource fork, and confirms the installation with
			the user, before calling through to JustInstallICComponent.
		*)
		var
			err: OSStatus;
			whereToInstall: FSSpec;
			prompt: Str255;
			junk: integer;
	begin
		err := noErr;
		UpdateInstalledComponentVersionGlobal(whereToInstall);
		(* gInstalledComponentVersion will be zero if there's no extension
			installed at all.
		*)
		if (gInstalledComponentVersion > 0) then begin
			if gInstalledComponentVersion < gApplicationVersionAsLongint then begin
				prompt := GetIndStr(rMiscStrings, strReplaceOlderExtension);
			end else if gInstalledComponentVersion > gApplicationVersionAsLongint then begin
				prompt := GetIndStr(rMiscStrings, strReplaceNewerExtension);
			end else begin
				prompt := GetIndStr(rMiscStrings, strReplaceCurrentExtension);
			end; (* if *)
			ParamText(prompt, '', '', '');
			InitCursor;
			if CautionAlert(rReplaceConfirmAlert, gOKCancelModalFilter) <> ditOK then begin
				err := userCanceledErr;
			end; (* if *)
		end; (* if *)
		if err = noErr then begin
			err := JustInstallICComponent;
		end; (* if *)
		if err = noErr then begin
			(* No matter what happens, you don't need to restart. Even if you're 
				installing an older version of the component, the IC application still
				has a connection to the newer version, so the application will
				remain happy. Of course it may complain the next time you launch
				it, but that's another issue.
			*)
			InitCursor;
			ParamText(GetIndStr(rMiscStrings, strNoNeedToRestart), '', '', '');
			junk := NoteAlert(rInstallResultAlert, nil);
		end; (* if *)
		InstallICComponent := err;
	end; (* InstallICComponent *)

	function RemoveICComponent: OSStatus;
		(* Implements the functionality of the Remove Extension command on
			the Install menu.
		*)
		var
			err: OSStatus;
			fss: FSSpec;
			junk: integer;
	begin
		err := noErr;
		UpdateInstalledComponentVersionGlobal(fss);
		if gInstalledComponentVersion > 0 then begin
			err := HDelete(fss.vRefNum, fss.parID, fss.name);
		end; (* if *)
		if err = noErr then begin
			if GetRegisteredComponentVersion > 0 then begin
				junk := NoteAlert(rRemoveResultAlert, nil);
			end; (* if *)
		end; (* if *)
		UpdateInstalledComponentVersionGlobal(fss);
		RemoveICComponent := err;
	end; (* RemoveICComponent *)

	procedure DoInstallMenu (item: integer);
	begin
		case item of
			iInstallExtension: 
				DisplayError(acInstallComponent, InstallICComponent);
			iSaveExtension: 
				DisplayError(acInstallComponent, SaveICComponent);
			iRemoveExtension: 
				DisplayError(acRemoveComponent, RemoveICComponent);
			otherwise
				;
		end; (* case *)
	end;

	(* ***** Initialisation Code ***** *)

	function ValidateInstalledICVersion: OSStatus;
		(* This routine examines the currently installed Internet Config component
			to see if we can run with it. There are basically three cases:
			1. component not installed -- return noErr 'cause we work just fine with glue.
			2. older version installed -- return error. IC application expects to have access
				to all the IC API routines that were defined at the time the application
				was built. If we have an older component installed, we may not have access
				to all these routines. Therefore we can't run with an older component.
			3. newer version installed -- return noErr, newer component is just fine.
		*)
		var
			err : OSStatus;
			err2: OSStatus;
			icComponentInstance: ComponentInstance;
			inst: ICInstance;
	begin
		err := ICStart(inst, ICcreator);
		if err = noErr then begin
			err := ICGetComponentInstance(inst, icComponentInstance);
			if err = noErr then begin
				(* Cases 2 and 3 go this way. *)
				if band(GetComponentVersion(icComponentInstance), $FFFF0000) < band(internetConfigurationComponentInterfaceVersion, $FFFF0000) then begin
					err := unimpErr;
				end; (* if *)
			end else begin
				(* Case 1 goes this way. *)
				err := noErr;
			end; (* if *)
			err2 := ICStop(inst);
			if err = noErr then begin
				err := err2;
			end; (* if *)
		end; (* if *)
		ValidateInstalledICVersion := err;
	end; (* ValidateInstalledICVersion *)

	function InitializeComponentInstallation: OSStatus;
		(* See comment in interface part. *)
		var
			fss: FSSpec;
			hitItem: integer;
			err: OSStatus;
	begin
		(* Setup the global version numbers. *)
		err := noErr;
		gApplicationVersionAsLongint := longint(gApplicationVersion.numericVersion);
		UpdateInstalledComponentVersionGlobal(fss);
		
		(* Don't do any of this stuff unless we have the Component Manager. *)
		if gHasComponentManager then begin
			if (gInstalledComponentVersion < gApplicationVersionAsLongint) then begin
				(* We don't have the required version of the extension. *)
				InitCursor;
				if gInstalledComponentVersion > 0 then begin
					(* There is already an extension present, ask whether we can update it. *)
					hitItem := NoteAlert(rUpdateConfirmAlert, gOKCancelModalFilter);
					if hitItem <> ok then begin
						err := userCanceledErr;
					end; (* if *)
				end else begin
					(* There is no extension, see if we're allowed to install it. *)
					ParamText(GetIndStr(rMiscStrings, strInstallExtensionPrompt), '', '', '');
					hitItem := NoteAlert(rInstallConfirmAlert, gOKCancelModalFilter);
				end; (* if *)
				if hitItem = ok then begin
					(* OK, go ahead and install the extension. *)
					err := JustInstallICComponent;
					DisplayError(acInstallComponent, err);
					if err = noErr then begin
						(* Now do a check to make sure we're using the new extension. *)
						err := ValidateInstalledICVersion;
						if err = noErr then begin
							(* Let the user know we don't need to restart. *)
							InitCursor;
							ParamText(GetIndStr(rMiscStrings, strNoNeedToRestart), '', '', '');
							hitItem := NoteAlert(rInstallResultAlert, nil);
						end else begin
							(* Let the user know we do need to restart. *)
							InitCursor;
							ParamText(GetIndStr(rMiscStrings, strYouNeedToRestart), '', '', '');
							hitItem := NoteAlert(rInstallResultAlert, nil);
							(* Return an error so the application will quit. *)
							err := userCanceledErr;
						end; (* if *)
					end; (* if *)
				end; (* if -- allowed to install extension *)
			end; (* if -- need to install extension *)
		end; (* if -- have Component Manager *)

		(* Finally, before we leave, we call ValidateInstalledICVersion again, just to make
			just that we're going to get the necessary API routines. This is just a paranoia
			check, there shouldn't be any way to get through the above sequence without
			having the required API routines, but sometimes it pays to be paranoid.
		*)
		if err = noErr then begin
			err := ValidateInstalledICVersion;
		end; (* if *)
		
		InitializeComponentInstallation := err;
	end; (* InitializeComponentInstallation *)

end. (* ICInstall *)
